﻿--[[

Game:	 Farming Simulator 19
Title:   Community Extension
Script:  AddUnits
Author:  ThundR
Created: 02/04/2021

Copyright ©2021, Todd Hundersmarck (ThundR)

Adding Units:
    - You can add units by using the unitManager:addUnit() function
    - You can remove units by commenting them out (precede line with --)
        + You can also remove units by simply deleting the line corresponding to that unit
    - Everything besides NAME and UNIT_TYPE can be adjusted in game
    - See below for more information

    - Example:
        + unitManager:addUnit(NAME, UNIT_TYPE, [FACTOR], [PRECISION], [TITLE], [TITLE_PLURAL], [TITLE_SHORT])

    - Parameters:
        + NAME: Internal unit name (e.g. "bushelUS")
            - This must be unique for each unit
        + UNIT_TYPE: Type of unit to create, available choices are as follows:
            - "volume": liter * factor --> new unit
            - "weight": tonne (liter * displayDensity) * factor --> new unit
            - "piece":  1:1 conversion, used for naming different "singular" units (e.g. pallets)
            - "animal": 1:1 conversion, used for naming different animal types (e.g. cows)
        + FACTOR: Unit conversion factor [optional]
            - Translates to how many new units are in one liter
            - This can be adjusted in-game
            - Defaults to 1.0 if omitted
            - Examples:
                * volume: 1.0 liters = 0.0283776 US bushels
                * weight: 1.0 tonnes = 1000 kilograms
            - Conversion site: https://www.thecalculatorsite.com/
        + PRECISION: Maximum number of decimal places to display [optional]
            - Certain screens use different values, this is max allowed precision
            - In most cases, variable precision is used, as follows:
                * 1.6548 with a variable precision of 4 = 1.6548
                * 100.6548 with a variable precision of 4 = 100.65
                * 10000.6548 with a variable precision of 4 = 10001
            - Defaults to 0 (no decimals) if omitted
        + TITLE: Optional in-game display name for a single unit (e.g. 1 bushel)
            - This can be adjusted in-game
            - This can also be added to included language files (see lang/lang_en.xml)
            - Defaults to NAME if omitted
        + TITLE_PLURAL: Optional in-game display name for multiple units (e.g. 12 bushels)
            - Can also be adjusted in-game as well as defined in language xml files
            - Defaults to NAME if omitted
        + TITLE_SHORT: Optional abbreviated in-game display name (e.g. 150 bu)
            - Can also be adjusted in-game as well as defined in language xml files
            - Defaults to NAME if omitted
            - This is always "pcs" for "piece" unit types
            - This is always blank (empty string) for "animal" unit types

Adding Density Units:
    - You can add density units by using the unitManager:addDensityUnit() function
    - Density units are used in menus to edit fillType density in units of choice
        + Game defaults to metric tons per liter (mt/l)
    - You can remove units by commenting them out (precede line with --)
        + You can also remove units by simply deleting the line corresponding to that unit
    - See below for more information

    - Example:
        + unitManager:addDensityUnit(NAME, WEIGHT_UNIT, VOLUME_UNIT, [PRECISION])

    - Parameters:
        + NAME: Internal unit name (e.g. "kgPerLiter")
            - This must be unique for each unit
        + WEIGHT_UNIT: Name of registered unit of weight (see above)
        + VOLUME_UNIT: Name of registered unit of volume (see above)
        + PRECISION: Maximum number of decimal places to display [optional]
            - Defaults to 0 if omitted

    - Density unit titles are handled by the specified volume and weight units (see above)

Language Files:
    - These can be found in the /lang folder
    - Each language file is named lang_[LANGUAGE_ABBREVIATION].xml
        + Example: English language is lang_en.xml (included with this mod)
    - Format is as follows: (each unit has three entries)
        <text name="cx_unit_[UNIT_NAME]"        text="[UNIT_TEXT]"/>
        <text name="cx_unit_[UNIT_NAME]_plural" text="[UNIT_TEXT]"/>
        <text name="cx_unit_[UNIT_NAME]_short"  text="[UNIT_TEXT]"/>
--]]

-- <<[Declarations]>> -----------------------------------------------------------------------------

local g_modName = g_currentModName
local g_modPath = g_currentModDirectory

---@class CXAddUnits
CXAddUnits = {}

-- <<[Initialization]>> ---------------------------------------------------------------------------

--[[ initialize -------------------------------]]
-- * called from CXUnitConvert.initialize()
-- = protected call
    ---@param parent CXUnitConvert @parent addon
    ---@param unitManager CXUnitConvert_UnitManager @unitManager environment
function CXAddUnits.initialize(parent, unitManager)
    -- add units here *******************************************************************

    -- titles are omitted in below registrations, as they are defined in lang/lang_en.xml
    -- you can add your own titles (see above) here, in the language xml files, or in-game

    -- if you are using an older version and want to keep your custom units,
    -- simply replace these addUnit entries with your custom ones from the older version

    unitManager:addUnit("bushelUS",   "volume", 0.0283775933,    2)
    unitManager:addUnit("gallonUS",   "volume", 0.2641720524,    1)
    unitManager:addUnit("cubicMeter", "volume", 0.0010000000,    2)
    unitManager:addUnit("cubicFoot",  "volume", 0.0353146667,    2)
    unitManager:addUnit("cubicYard",  "volume", 0.0013079506,    2)

    unitManager:addUnit("tonne",      "weight", 1.0000000000,    4)
    unitManager:addUnit("tonUS",      "weight", 1.1023113109,    4)
    unitManager:addUnit("kilogram",   "weight", 1000.0000000000, 2)
    unitManager:addUnit("pound",      "weight", 2204.6226218488, 2)

    unitManager:addUnit("bale",       "piece")

    -- add density units here ***********************************************************

    -- if you are using an older version and want to keep your custom units,
    -- simply replace these addDensityUnit entries with your custom ones from the older version

    -- default weight unit is tonne
    -- default volume unit is liter

    unitManager:addDensityUnit("mtPerLiter",   "tonne",    "liter",      6) -- vanilla format
    unitManager:addDensityUnit("kgPerLiter",   "kilogram", "liter",      4)
    unitManager:addDensityUnit("kgPerM3",      "kilogram", "cubicMeter", 0)
    unitManager:addDensityUnit("lbsPerFt3",    "pound",    "cubicFoot",  2)
    unitManager:addDensityUnit("lbsPerBushel", "pound",    "bushelUS",   2)
    unitManager:addDensityUnit("lbsPerGallon", "pound",    "gallonUS",   2)
    unitManager:addDensityUnit("tonPerBushel", "tonUS",    "bushelUS",   4)
    unitManager:addDensityUnit("tonPerGallon", "tonUS",    "gallonUS",   4)
end
